#region License and Credits
/* =====================================================================================================
 * Original Creator: Alan. L. Hunter.
 * -----------------------------------------------------------------------------------------------------
 * License: You may use this code free of charge as long as this license is left intact and unedited.
 * Any changes you make to the original source code must be documented and clearly marked. Credit to
 * myself must be given in the about box of your program, or some other easily accessable place inside
 * the program.
 * =====================================================================================================
 * Changes - This is where you document any changes made to the source code
 * =====================================================================================================
 */
#endregion

#region Using Statements
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Xml;

using EQIA.Manager;
#endregion

namespace EQ_Ini_Doctor
{
    public partial class MainForm : Form
    {
        #region Members

        /// <summary>
        /// Path of the INI file.
        /// </summary>
        private string myIniPath;

        /// <summary>
        /// A list of screens in the Custom UI.
        /// </summary>
        private List<EQIA.EQIDL.Screen> myCustomUi;

        /// <summary>
        /// The INI file without any modifications.
        /// </summary>
        private List<string> myIniBeforeChanges;

        /// <summary>
        /// The INI file after modifications.
        /// </summary>
        private List<string> myIniAfterChanges;

        /// <summary>
        /// A list of changes to perform on the INI file.
        /// </summary>
        private List<string> myIniChanges;

        #endregion

        #region Constructor

        public MainForm()
        {
            InitializeComponent();

            myCustomUi = new List<EQIA.EQIDL.Screen>();

            myIniBeforeChanges = new List<string>();
            myIniAfterChanges = new List<string>();
            myIniChanges = new List<string>();

            myIniPath = "";
        }

        #endregion

        #region Methods

        /// <summary>
        /// Parses a Custom UI, adding Screen elements to the Screen Element list.
        /// </summary>
        /// <param name="path">Path to the Custom UI folder.</param>
        private void ParseCustomUi(string path)
        {
            // Recreate the Custom UI Screen list
            myCustomUi = new List<EQIA.EQIDL.Screen>();

            // Directory info about the Custom UI directory
            DirectoryInfo dir = new DirectoryInfo(path);

            // A list of files in the Custom UI directory
            FileInfo[] files = dir.GetFiles("*.xml");

            // Loop through the files in the Custom UI
            for (int i = 0; i < files.Length; i++)
            {
                // An XML file to store the current XMLs data in
                XmlDocument xml = new XmlDocument();

                // Load the XML file
                xml.Load(files[i].FullName);

                // Grab the root node of the XML file
                XmlNode xnRoot = xml.DocumentElement;

                // Loop through the child nodes of the root element
                for (int j = 0; j < xnRoot.ChildNodes.Count; j++)
                {
                    // If the current child node is a Screen element
                    if (xnRoot.ChildNodes[j].NodeType == XmlNodeType.Element && 
                        xnRoot.ChildNodes[j].LocalName == "Screen")
                    {
                        // Create a new screen that will hold the data
                        EQIA.EQIDL.Screen newScreen = new EQIA.EQIDL.Screen();

                        // Load the XML of the current Screen element into the Screen class
                        newScreen.LoadXml(xnRoot.ChildNodes[j].OuterXml);

                        // Deserialize the XML so that the Screen class has data to check
                        newScreen.Deserialize();

                        // Attempt to add the Screen to the list
                        try
                        {
                            myCustomUi.Add(newScreen);
                        }
                        catch 
                        { 
                            // Don't worry about throwing an error, the Screens that count won't fail
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Parse a UI_Player_Server.ini file, and add it's contents to the INI string List.
        /// </summary>
        /// <param name="path">Path to the INI file.</param>
        private void ParsePlayerIni(string path)
        {
            // Recreate the INI string list
            myIniBeforeChanges = new List<string>();
            myIniAfterChanges = new List<string>();

            // Retrieve information about the INI file
            FileInfo myIniInfo = new FileInfo(path);

            // If the file actually exist
            if (myIniInfo.Exists)
            {
                // Set the INI path to match the fed in path
                myIniPath = path;

                // Read through the file and add each line to the list
                using (StreamReader reader = new StreamReader(myIniInfo.FullName))
                {
                    string line = "";

                    while ((line = reader.ReadLine()) != null)
                        myIniBeforeChanges.Add(line);
                }
            }

            // Copy the list over so we have something to change
            myIniAfterChanges = myIniBeforeChanges;
        }

        #endregion

        #region Event Handlers

        private void btn_LoadCustomUI_Click(object sender, EventArgs e)
        {
            // Set the filter to XML files
            myOpenFileDialog.Filter = "XML|*.xml";
            myOpenFileDialog.FileName = "";

            if (myOpenFileDialog.ShowDialog() == DialogResult.OK)
            {
                txb_CustomUI.Text = "Parsing Custom UI, Please Wait...";

                // Trim off the actual file name
                string path = myOpenFileDialog.FileName.Substring(0, myOpenFileDialog.FileName.LastIndexOf('\\') + 1);

                // Parse the UI
                ParseCustomUi(path);

                txb_CustomUI.Text = path;
            }
        }

        private void btn_LoadIni_Click(object sender, EventArgs e)
        {
            // Set the filter to UI INI files
            myOpenFileDialog.Filter = "Player UI INI|UI*.ini";
            myOpenFileDialog.FileName = "";

            if (myOpenFileDialog.ShowDialog() == DialogResult.OK)
            {
                txb_PlayerIni.Text = "Parsing Player INI, Please Wait...";

                // Parse the INI file
                ParsePlayerIni(myOpenFileDialog.FileName);

                txb_PlayerIni.Text = myOpenFileDialog.FileName;
            }
        }

        private void btn_Process_Click(object sender, EventArgs e)
        {
            // Loop through each screen in the custom UI
            for (int i = 0; i < myCustomUi.Count; i++)
            {
                // An index to mark the section of the INI this screen pertains to
                int indexOfSection = myIniBeforeChanges.IndexOf("[" + myCustomUi[i].item + "]");

                // An index to mark the next section
                int indexOfNextSection = indexOfSection;

                // Flags to keep track of wheather width and height were found
                bool wFound = false;
                bool hFound = false;

                // If the index is -1, it don't exist in the INI
                if (indexOfSection != -1)
                {
                    string section = "[" + myCustomUi[i].item + "]";

                    // Loop through the file until it hits the next section and take note of it
                    for (int j = indexOfSection + 1; j < myIniBeforeChanges.Count; j++)
                    {
                        if (myIniBeforeChanges[j].StartsWith("[") && myIniBeforeChanges[j].EndsWith("]"))
                        {
                            indexOfNextSection = j;
                            break;
                        }
                    }

                    // Loop through each line of the INI to find the 'Width' and 'Height' properties
                    for (int j = indexOfSection + 1; j < indexOfNextSection; j++)
                    {
                        // If the section before the '=' is 'Width', we've found the 'Width' property
                        if (myIniBeforeChanges[j].Split('=')[0] == "Width")
                        {
                            // If the width of the custom UI window is 'bigger than' the entry
                            if (myCustomUi[i].Size.Width > System.Convert.ToInt32(myIniBeforeChanges[j].Split('=')[1]))
                            {
                                myIniChanges.Add(
                                    "Action: Change, " +
                                    "Section: " + section + ", " +
                                    "Old: " + myIniBeforeChanges[j] + ", " +
                                    "New: Width=" + myCustomUi[i].Size.Width.ToString());
                            }

                            wFound = true;
                        }

                        // If the section before the '=' is 'Height', we've found the 'Height' property
                        if (myIniBeforeChanges[j].Split('=')[0] == "Height")
                        {
                            // If the height of the custom UI window is 'bigger than' the entry
                            if (myCustomUi[i].Size.Height > System.Convert.ToInt32(myIniBeforeChanges[j].Split('=')[1]))
                            {
                                myIniChanges.Add(
                                    "Action: Change, " +
                                    "Section: " + section + ", " +
                                    "Old: " + myIniBeforeChanges[j] + ", " +
                                    "New: Height=" + myCustomUi[i].Size.Height.ToString());
                            }

                            hFound = true;
                        }

                        // If this is the last line of the section, check if width and height was found
                        if (j == indexOfNextSection - 1)
                        {
                            if (!wFound)
                            {
                                myIniChanges.Add(
                                    "Action: Add, " +
                                    "Section: " + section + ", " +
                                    "New: Width=" + myCustomUi[i].Size.Width.ToString());
                            }

                            if (!hFound)
                            {
                                myIniChanges.Add(
                                    "Action: Add, " +
                                    "Section: " + section + ", " +
                                    "New: Height=" + myCustomUi[i].Size.Height.ToString());
                            }
                        }
                    }
                }
            }

            // Update the list box with the list of changes
            lb_Changes.Items.Clear();
            for (int i = 0; i < myIniChanges.Count; i++)
            {
                lb_Changes.Items.Add(myIniChanges[i]);
            }
        }

        private void btn_ApplyChanges_Click(object sender, EventArgs e)
        {
            // Loop through the list of changes
            for (int i = 0; i < myIniChanges.Count; i++)
            {
                string[] changes = myIniChanges[i].Split(',');

                // Action to take
                string action = changes[0].Split(':')[1].Trim();

                // Section to take action on
                string section = changes[1].Split(':')[1].Trim();

                // Index of the section
                int changeIndex = myIniAfterChanges.IndexOf(section);

                // If the index was found, proceed
                if (changeIndex != -1)
                {
                    // Perform whichever action was noted
                    switch (action)
                    {
                        case "Add":
                            {
                                myIniAfterChanges.Insert(changeIndex + 2, changes[2].Split(':')[1].Trim());
                                lb_Changes.Items[i] = "Applied";
                            }
                            break;

                        case "Change":
                            {
                                string oldEntry = changes[2].Split(':')[1].Trim();
                                string newEntry = changes[3].Split(':')[1].Trim();

                                for (int j = changeIndex + 2; j < myIniAfterChanges.Count; j++)
                                {
                                    if (myIniAfterChanges[j].StartsWith("[") && myIniAfterChanges[j].EndsWith("]"))
                                    {
                                        lb_Changes.Items[i] = "Skipped";
                                        break;
                                    }

                                    if (myIniAfterChanges[j] == oldEntry)
                                    {
                                        myIniAfterChanges[j] = newEntry;
                                        lb_Changes.Items[i] = "Applied";
                                    }
                                }
                            }
                            break;
                    }
                }
            }

            // Write the changed data to the INI file
            using (TextWriter tw = new StreamWriter(myIniPath))
            {
                for (int i = 0; i < myIniAfterChanges.Count; i++)
                    tw.WriteLine(myIniAfterChanges[i]);
            }
        }

        #endregion
    }
}